/***************************************************************************
                           main.cpp
                             -------------------
    begin                : Don Apr 17 11:30:44 CEST 2003
    revision             : $Revision: 1.28 $
    last modified        : $Date: 2005/07/10 13:12:14 $ by $Author: juergenk $
    copyright            : (C) 2003-2005 by Jürgen Kofler
    email                : kaffeine@gmx.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "kaffeineapp.h"

#include <kcmdlineargs.h>
#include <kaboutdata.h>
#include <klocale.h>
#include <kdebug.h>

#include <X11/Xlib.h>

#if defined SOLARIS
#include <priv.h>
static priv_set_t* privset = 0L;
#endif

#include <stdlib.h>
#include <unistd.h>
#include <strings.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/resource.h>
#include <pthread.h>
#include <errno.h>

extern "C" {
    static void setSystemLimits (void)
    {
        static struct rlimit maxLimit;
        (void) memset (&maxLimit, '\0', sizeof (struct rlimit));

	if (getrlimit (RLIMIT_NOFILE, &maxLimit) < 0)
	{
	    perror ("getrlimit"), strerror (errno);
	    return;
	}

	if (maxLimit.rlim_cur < 1024)
	    maxLimit.rlim_cur = 1024;
	if (maxLimit.rlim_max < 8192)
	    maxLimit.rlim_max = 8192;

        if (setrlimit (RLIMIT_NOFILE, &maxLimit) < 0)
            perror ("setrlimit"), strerror (errno);
        return;
    }         
}
  
extern "C"
{
    typedef void SignalHandlerFunction (int);
    static SignalHandlerFunction* KSignal (int signo, SignalHandlerFunction* sigFunc)
    {
        static struct sigaction act;
	static struct sigaction oact;
        act.sa_handler = sigFunc;
        sigemptyset (&act.sa_mask);
        act.sa_flags = 0;
        if (signo == SIGALRM)
        {
#ifdef SA_INTERRUPT
            act.sa_flags |= SA_INTERRUPT;
#endif
        }
        else
        {
            act.sa_flags |= SA_RESTART;
        }
        if (sigaction (signo, &act, &oact) != 0)
                return (SIG_ERR);
        return (oact.sa_handler);
    }
}

extern "C"
{
    static void KSignalHandler (int signo)
    {
	KSignal (signo, SIG_IGN);
	KSignal (signo, KSignalHandler);
    }
}

extern "C"
{
    static void set_privileges(void)
    {
	privset = priv_allocset();
	if (0L == privset)
	    return;

	if (getppriv(PRIV_PERMITTED, privset) != 0)
	{
	    perror("getppriv"), strerror(errno);
	    return;
	}
	if (priv_addset(privset, "PRIV_SYS_DEVICES") != 0)
	{
	    perror("priv_addset"), strerror(errno);
	    return;
	}
	if (setppriv(PRIV_ON, PRIV_EFFECTIVE, privset) != 0)
	    perror("setppriv"), strerror(errno);
    }
}

static KCmdLineOptions options[] =
{
  { "p", 0, 0 },
  { "play", I18N_NOOP ("Start playing immediately"), 0 },
  { "f", 0, 0 },
  { "fullscreen", I18N_NOOP ("Start in fullscreen mode"), 0 },
  { "a", 0, 0 },
  { "audiodriver <argument>", I18N_NOOP ("Set audio driver"), "default" },
  { "x", 0, 0 },
  { "videodriver <argument>", I18N_NOOP ("Set video driver"), "default" },
  { "d", 0, 0 },
  { "device <argument>", I18N_NOOP ("Set Audio-CD/VCD/DVD device path."), "default"},
  { "", 0, 0 },
  { "verbose", I18N_NOOP ("Output xine debug messages"), 0 },
  { "w", 0, 0 },
  { "wizard", I18N_NOOP ("Run installation wizard"), 0 },
  { "+[file]", I18N_NOOP ("File(s) to play. Can be a local file, a URL, a directory or 'DVD', 'VCD', 'AudioCD'."), 0 },
  KCmdLineLastOption
};


int main(int argc, char *argv[])
{
  /*  XInitThreads() should always be the first call in multithreaded X programs -
   *  but because of a bug in some xfree versions that can freeze at startup,
   *  we also call XInitThreads() in kxinewidget.cpp
   *
   *  kaffeinepart & konqueror: We call XInitThreads() in kxinewidget.cpp, so the part is
   *  not stable with buggy X implementations.
   */

#if defined SOLARIS
  set_privileges();
  KSignal(SIGPIPE, KSignalHandler);
  KSignal(SIGALRM, KSignalHandler);
  pthread_setconcurrency(static_cast<int>(::sysconf(_SC_NPROCESSORS_ONLN * 2)));
#endif

#ifndef XINIT_WKRND  // configure flag --with-xinit-workaround not set
  kdDebug() << "Main: XInitThreads()" << endl;
  kdDebug() << "[INFO] If Kaffeine hangs here run 'configure --with-xinit-workaround' and recompile/reinstall." << endl;
  XInitThreads();
#endif

  KAboutData aboutData( "kaffeine", I18N_NOOP("Kaffeine Player"),
    VERSION, I18N_NOOP("A media player for KDE 3. Can use multiple backends for playback, default is xine."),
    KAboutData::License_GPL,
    I18N_NOOP("(c) 2003-2005, The Kaffeine Authors"), 0, "http://kaffeine.sourceforge.net", /*bug reports to:*/ "kaffeine-user@lists.sourceforge.net");

  aboutData.addAuthor("Jürgen Kofler", I18N_NOOP("Developer"), "kaffeine@gmx.net");
  aboutData.addAuthor("Christophe Thommeret", I18N_NOOP("DVB Support"), "hftom@free.fr");

  aboutData.addCredit("Anders Ellenshøj Andersen", I18N_NOOP("Logo animation for Kaffeine 0.5"), "andersa@ellenshoej.dk");
  aboutData.addCredit("Assaf Gillat", I18N_NOOP("Alternate encoding for meta tags. Many patches."), "gillata@gmail.com");
  aboutData.addCredit("Miguel Freitas", I18N_NOOP("xine post plugin handling. Many patches."), "miguel@cetuc.puc-rio.br", "http://www.xinehq.de");
  aboutData.addCredit("Giorgos Gousios", I18N_NOOP("Subtitle file import."), "gousiosg@cs.man.ac.uk");
  aboutData.addCredit("Michael Rolf", I18N_NOOP("M3U import. Testing."), "mi.rolf@gmx.net");


  KCmdLineArgs::init( argc, argv, &aboutData );
  KCmdLineArgs::addCmdLineOptions( options );

  if (!KaffeineApp::start())
  {
    exit (0);    /* can't start new instance */
  }

  KaffeineApp* a = new KaffeineApp();;
  int ret = a->exec();
  delete a;
  return ret;
}


